@@ -16,51 +16,84 @@ import numpy as np
 import tempfile 
 
 #DOCU : 
-# MASK 4D X Y Z T , T = 1 OU PLUS. CHAQUE SLICE NE DOIT PAS AVOIR MOINS DE 3 PIXELS ISOLES
-#(cf clean_mask in tools)
+# MASK : NDARRAY 3D [Z, X, Y] OR SITK IMG (X, Y, Z) => CHAQUE SLICE NE DOIT PAS AVOIR MOINS DE 3 PIXELS ISOLES
+#(cf clean argument)
 #DICT : generate_dict in tools : PAS PLUS DE 16 CARACTERES 
 class RTSS_Writer:
     """A class for DICOM RT format
     """
     
-    def __init__(self, mask, serie_path):
+    def __init__(self, mask, serie_path, mask_mode, clean = True, sous_seg = True):
         """[summary]
 
         Args:
-            mask ([array]): [4D array [x,y,z,t], if ROI/slice with 1 or 2 pixels, replace by background value = 0]
+            mask ([ndarray or sitk_img]): []
             serie_path ([str]): [Serie path related to RTSTRUCT file ]
+            mask_mode ([str]) : ['img' if a sitk_img or 'matrix' if ndarray]
+            clean ([bool]) : [Clean the mask or not. Set to True]
+            label ([bool]) : [Transform binary mask into labelled mask. Set to True]
         """
-        self.mask = mask
 
-        #data spécifique à la série ou on dessine les contours 
+        self.mask = mask
+        self.mask_mode = mask_mode 
+        self.clean = clean 
+        self.sous_seg = sous_seg
+
         serie = Series(serie_path)
         self.instances = serie.get_instances_ordered()
-        self.image_position = self.instances[0].get_image_position()
-        self.pixel_spacing = self.instances[0].get_pixel_spacing()
-        self.pixel_spacing.append(serie.get_z_spacing())
-        
+
+
+        if mask_mode == 'matrix' : 
+            self.mask_array = mask # 3D ndarray [z, x, y]
+            self.mask_img = sitk.GetImageFromArray(self.mask_array)
+            #data spécifique à la série ou on dessine les contours : origin, spacing, direction 
+
+            self.image_position = self.instances[0].get_image_position() #origin 
+            self.pixel_spacing = self.instances[0].get_pixel_spacing()
+            self.pixel_spacing.append(abs(serie.get_z_spacing())) #spacing 
+            original_direction = self.instances[0].get_image_orientation()
+            self.image_direction = (float(original_direction[0]), float(original_direction[1]), float(original_direction[2]), 
+                                        float(original_direction[3]), float(original_direction[4]), abs(float(original_direction[5])), 
+                                        0.0, 0.0, 1.0) 
+            self.mask_img.SetSpacing(self.pixel_spacing)
+            self.mask_img.SetOrigin(self.image_position)
+            self.mask_img.SetDirection(self.image_direction)
+
+        elif mask_mode == 'img' : 
+            self.mask_img = mask
+            self.mask_array = read_sitk_img(self.mask_img) # 3Dndarray [z,x,y]
+            self.image_position = self.mask_img.GetOrigin()
+            self.pixel_spacing = self.mask_img.GetSpacing()
+            self.image_direction = self.mask_img.GetDirection()
+
+        if clean == True : 
+            self.mask_array = clean_mask(self.mask_array)
+
+        if sous_seg == True : 
+            #Ici implémenter watershed 
+            self.mask_array = label(self.mask_array, connectivity=2)
+
+
+
+        #get number of ROI after label
+        self.number_of_roi = int(np.max(self.mask_array))
+
+
+
         #Get list of every sop instance uid 
         self.list_all_SOPInstanceUID = get_list_SOPInstance_UID(self.instances)
-    
-
-        #str to float 
-        for i in range(len(self.image_position)):
-            self.image_position[i] = float(self.image_position[i])
-            self.pixel_spacing[i] = float(self.pixel_spacing[i])
 
         #generate dictionnary with paramter inside 
         self.generate_dict_json()
-        #print(self.results)
 
         #creation file_meta 
         self.file_meta, self.file_meta.MediaStorageSOPInstanceUID = self.generates_file_meta()
-        #creation fileDataset with value inside 
         suffix = '.dcm'
         filename_little_endian = tempfile.NamedTemporaryFile(suffix=suffix).name
         self.dataset = pydicom.dataset.FileDataset(filename_little_endian, {}, file_meta=self.file_meta, preamble=b"\0" * 128)
 
         #Add the data element in FileDataset
-        self.set_tags(serie_path)
+        self.set_tags()
         self.set_StructureSetROISequence()
         self.set_RTROIObservationSequence()
         self.set_ROIContourSequence()
@@ -71,8 +104,8 @@ class RTSS_Writer:
         self.dataset.is_implicit_VR = False 
 
     def generate_dict_json(self):
-        number_of_roi = get_number_of_roi(self.mask)
-        results = generate_dict(number_of_roi)
+        number_of_roi = get_number_of_roi(self.mask_array)
+        results = generate_dict(number_of_roi, 'rtstruct')
         self.results = results
         return None 
 
@@ -102,7 +135,7 @@ class RTSS_Writer:
         return file_meta, file_meta.MediaStorageSOPInstanceUID
 
 
-    def set_tags(self,serie_path):
+    def set_tags(self):
         """
             
             List of tags :
@@ -164,18 +197,18 @@ class RTSS_Writer:
     #StructureSetROISequence
     def set_StructureSetROISequence(self):
         referenced_frame_of_reference_uid = self.instances[0].get_frame_of_reference_uid()
-        self.dataset.StructureSetROISequence = StructureSetROISequence(self.mask, self.results).create_StructureSetROISequence(self.pixel_spacing, referenced_frame_of_reference_uid)
+        self.dataset.StructureSetROISequence = StructureSetROISequence(self.mask_array, self.results, self.number_of_roi).create_StructureSetROISequence(self.pixel_spacing, referenced_frame_of_reference_uid)
         
 
     #RTROIObservationSequence
     def set_RTROIObservationSequence(self):
-        self.dataset.RTROIObservationsSequence = RTROIObservationsSequence(self.mask, self.results).create_RTROIObservationsSequence()
+        self.dataset.RTROIObservationsSequence = RTROIObservationsSequence(self.results, self.number_of_roi).create_RTROIObservationsSequence()
         
 
     #ROIContourSequence 
     def set_ROIContourSequence(self):
         referenced_sop_class_uid = self.instances[0].get_sop_class_uid()
-        self.dataset.ROIContourSequence = ROIContourSequence(self.mask).create_ROIContourSequence(referenced_sop_class_uid, self.image_position, self.pixel_spacing, self.list_all_SOPInstanceUID, self.instances)
+        self.dataset.ROIContourSequence = ROIContourSequence(self.mask_array, self.mask_img, self.number_of_roi).create_ROIContourSequence(referenced_sop_class_uid, self.list_all_SOPInstanceUID, self.instances)
 
     #ReferencedFrameOfReferenceSequence 
     def set_ReferencedFrameOfReferenceSequence(self):
