@@ -4,9 +4,9 @@ from library_dicom.export_segmentation.tools.rtss_writer_tools import *
 
 class StructureSetROISequence : 
 
-    def __init__(self, mask, results):
-        self.mask = mask
-        self.number_of_roi = get_number_of_roi(self.mask)
+    def __init__(self, mask_array, results, number_of_roi):
+        self.mask_array = mask_array
+        self.number_of_roi = number_of_roi
         self.results = results
 
 
@@ -16,18 +16,9 @@ class StructureSetROISequence :
         for number_roi in range(1 , self.number_of_roi +1):
             dataset = pydicom.dataset.Dataset()
             dataset.ROINumber = number_roi
-            dataset.ReferencedFrameOfReferenceUID = ReferencedFrameOfReferenceUID 
-            if self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"] != "None": 
-                dataset.ROIName = self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"]
-
-            else : dataset.ROIName = str(number_roi)
-
+            dataset.ReferencedFrameOfReferenceUID = ReferencedFrameOfReferenceUID             
+            dataset.ROIName = self.results['segmentAttributes'][0][number_roi-1]["SegmentDescription"]
             dataset.ROIVolume = str(self.get_roi_volume(number_roi, pixel_spacing))
-                
-             
-            #if self.dict_roi_data[number_roi]['ROIGenerationAlgorithm'] != '' :
-            #    dataset.ROIGenerationAlgorithm = self.dict_roi_data[number_roi]['ROIGenerationAlgorithm']
-            #else : dataset.ROIGenerationAlgorithm = 'UNDEFINED'
             dataset.ROIGenerationAlgorithm = self.results['segmentAttributes'][0][number_roi-1]["SegmentAlgorithmType"]
             StructureSetROISequence.append(dataset)
         return StructureSetROISequence 
@@ -35,13 +26,10 @@ class StructureSetROISequence :
      
     def get_roi_volume(self, number_roi, pixel_spacing):
         number_pixel = 0
-
-        if len(self.mask.shape) == 3 :
-             x = np.where(self.mask == number_roi)[0]
-        elif len(self.mask.shape) == 4 : 
-            if self.mask.shape[-1] != 1 : 
-                x = np.where(self.mask[:,:,:,number_roi-1] != 0)[0]
-            else : x = np.where(self.mask == number_roi)[0]
+        if len(self.mask_array.shape) == 3 :
+            _, x, _ = np.where(self.mask_array == number_roi)
+        elif len(self.mask_array.shape) == 4 : 
+            raise Exception ('4D array. Need a 3D array with [z,x,y]')
         number_pixel = len(x) #same as len(y) or len(z)
         volume_pixel = pixel_spacing[0] * pixel_spacing[1] * abs(pixel_spacing[2])
         volume_pixel = volume_pixel * 10**(-3) #mm3 to ml 
